#pragma once
#ifndef _PO_Assert_
#define _PO_Assert_
/////////////////////////////////////////////////////////////////////////////////////////////
//
//	LIBRARY:		PO_Assert
//
//	DESCRIPTION:	Provides some basic assert reporters
//
//	
//	Testing assertions
//	------------------
//
//		assert(expression)				// same as PO_Assert
//		PO_Assert(expression)			// (PO_ASSERT_FATAL ? PO_AssertFatal(expression) : PO_AssertInform(expression) )
//		PO_AssertFatal(expression)		// reports a message indicating Fatal error  (see below)
//		PO_AssertInform(expression)		// reports a message indicating informitive error (see below)
//			
//	How messages are reported
//	-------------------------
//	
//		PO_ASSERT_REPORT	defines how messages are reported.  The options are as followes
//		
//			PO_ASSERT_REPORT_OFF		-- off regardless of NDEBUG
//			PO_ASSERT_REPORT_MACSBUG	-- Does a PO_DebugUtils::DebugStr
//			PO_ASSERT_REPORT_COUT		-- Does a cout << message
//			PO_ASSERT_REPORT_EXTERNAL	-- calls an external function PO_Assert_Report that you must
//										   define.
//										   
//		The default is PO_ASSERT_REPORT_MACSBUG to be backward compatible with assertmb
//		
//		NOTE:  Note that these are compile time options, NOT runtime options.
//			   This is so we don't have to link in extra baggage that we are
//			   not using.
//	
//	What is the difference between "Fatal" and "inform"
//	---------------------------------------------------
//	
//		Normal usage of asserts are of a "Fatal" nature.  We use asserts to let us know that
//		something that should never happen just did.  Normally it is expected that program
//		termination will result from a fatal assertion.
//		
//		For convience, I've introduced the concept of a "non-fatal" or "informitive" assertion.
//		This is an assertion that says, "hey you may want to know this, but I can keep running".
//		"Memory is getting low" could be a good example of an informative assertion.
//		
//		The "assertion reporter" determines the difference between a fatal and an informative assertion.
//		
//		For the built in "macsbug" reporter, a fatal assertion stops, an informative assertion
//		does a ';g' after the message.
//		
//		For the built in "cout" reporter, a fatal assertion calls exit(), whereas an
//		informative assertion just continues on after reporting the message
//		
//		For an "external reporter", you decide what to do for a fatal assertion.
//		
//
//		NOTE:	The default "mode" for PO_Assert and assert is FATAL.
//				You can change this by redefining PO_ASSERT_FATAL to be false
//				
//			
/////////////////////////////////////////////////////////////////////////////////////////////
		

/////////////////////////////////////////////////////////////////////////////////////////////
//	REPORTING OPTIONS
/////////////////////////////////////////////////////////////////////////////////////////////

#define PO_ASSERT_REPORT_OFF			0		// off, regardless of NDEBUG
#define PO_ASSERT_REPORT_MACSBUG		1		// PO_DebugUtils::DebugStr	(safe)  (fatal stops, inform does ';g')
#define PO_ASSERT_REPORT_COUT			2		// cout << message  (fatal calls exit, inform does nothing)
#define PO_ASSERT_REPORT_EXTERNAL		3		// you provide PO_Assert_Report implementation


#ifndef PO_ASSERT_REPORT
#define PO_ASSERT_REPORT	PO_ASSERT_REPORT_MACSBUG	// Default is macsbug
#endif


/////////////////////////////////////////////////////////////////////////////////////////////
//	FATAL OPTIONS
/////////////////////////////////////////////////////////////////////////////////////////////


#ifndef PO_ASSERT_FATAL
#define PO_ASSERT_FATAL		true						// default is fatal
#endif


/////////////////////////////////////////////////////////////////////////////////////////////
//	DEFINITIONS
/////////////////////////////////////////////////////////////////////////////////////////////

#undef PO_Assert

#if PO_ASSERT_REPORT==PO_ASSERT_REPORT_OFF || defined(NDEBUG)

	#define PO_Assert(x)		((void) 0)
	#define PO_Assert2(x,y)		((void) 0)
	#define PO_AssertFatal(x)	((void) 0)
	#define PO_AssertInform(x)	((void) 0)

#else

	// For PO_ASSERT_REPORT_EXTERNAL, you define this function
	
	void	PO_Assert_Report(ConstStr255Param inMessage,bool inFatalQ);		// reporting function

	// private function
	void 	__po_assert(const char* inAssert, const char* inFile, long inLine,bool inFatal);
	void	__po_assert2(const char* inAssert,const char* inFile, long inLine,long inValue,bool inFatal);

	#define PO_Assert(x)		((x) ? ((void)0) : ((void) __po_assert(#x,__FILE__,__LINE__,PO_ASSERT_FATAL)) )
	#define PO_Assert2(x,y)		((x) ? ((void)0) : ((void) __po_assert2(#x,__FILE__,__LINE__,(long)y,PO_ASSERT_FATAL)) )
	#define PO_AssertFatal(x)	((x) ? ((void)0) : ((void) __po_assert(#x,__FILE__,__LINE__,true)) )
	#define PO_AssertInform(x)	((x) ? ((void)0) : ((void) __po_assert(#x,__FILE__,__LINE__,false)) )
	
#endif


#undef 	assert
#define assert	PO_Assert

#define assert2 PO_Assert2

#endif

