#pragma once
#ifndef INCLUDED_PO_GLOBALS
#define INCLUDED_PO_GLOBALS

//////////////////////////////////////////////////////////////////////////
//
//	HEADER:		PO_Globals
//	
//	PURPOSE:	For 68K compilers, a register is used to reference
//				global variables.  This header file helps in two
//				ways.
//				
//				1.	We hide the SetA5 stuff using the macros below
//					so that if we compile as a code resource, we
//					can just change the macros and things will be ok
//					for a4 code.
//					
//				2.	Since different compilers use different
//					ways of setting up a4 for code resources, we
//					can hide that here.
//					
//
//	USAGE:		This library correctly defines the following
//				macros for A4, A5, or CFM globals
//				for Metrowerks and Symantec compilers
//				
//				Some macros have two names (old and new)
//
//				NEW NAME				OLD NAME			DESCRIPTION
//				--------				--------			-----------
//				PO_EnterGlobals			EnterCodeResource	Initializes the global register
//															on entry to a code module
//															(ie, code resource)
//
//				PO_ExitGlobals			ExitCodeResource	Resets the global register
//															on exit of a code module
//															(ie, code resource)
//															
//				PO_PrepareCallback		PrepareCallback		Call this before a call back
//															function is entered in your code
//															module.  (ie, time manager task, etc..)
//
//				PO_EnterCallback		EnterCallback		Call this on entry to your callback
//
//				PO_ExitCallback			ExitCallback		Call this on exit of your callback
//
//				PO_GetGlobals								Gets the value of the global register
//				
//				PO_SetGlobals(x)							Sets the value of the global register
//
//				__G_A4__									defined if A4 globals are used
//				__G_A5__									defined if A5 globals are used
//				__G_CFM__									defined if CFM globals are used
//																(CFM NOT PPC, could be 68KCFM)
//
//	AUTHORS:	Bill Hubauer (BH)
//	
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		1/95		BH		Created for application support only.
//		8/95		BH		Added support for a4
//		9/95		BH		Added Symantec Support
//		12/97		BH		Updated for PO lib
//		
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
//	QUICK REFERENCE
//		See "Usage" above for more info
//////////////////////////////////////////////////////////////////////////

/*
	The following macros are defined
	
	PO_EnterGlobals()
	PO_ExitGlobals()
	
	PO_PrepareCallback()
	PO_EnterCallback()
	PO_ExitCallback()
	
	PO_GetGlobals()
	PO_SetGlobals(x)
	
	EnterCodeResource()
	ExitCodeResource()
	
	PrepareCallback()
	EnterCallback()
	ExitCallback()
	
	__G_A4__
	__G_A5__
	__G_CFM__
*/


//////////////////////////////////////////////////////////////////////////
//	INCLUDES
//////////////////////////////////////////////////////////////////////////

#include <ConditionalMacros.h>

//////////////////////////////////////////////////////////////////////////
//	DETERMINE WHICH REGISTER REFERENCES GLOBALS (A4, A5, CFM)
//////////////////////////////////////////////////////////////////////////

#undef __G_A5__
#undef __G_A4__
#undef __G_CFM__

#if GENERATINGCFM
	#define __G_CFM__
#else
#ifdef __MWERKS__
	#if __MC68K__
		#if __A5__
			#define __G_A5__
		#else
			#define __G_A4__
		#endif
	#endif
#else
#ifdef __SC__
	#if __A4_GLOBALS__
		#define __G_A4__
	#else
		#define __G_A5__
	#endif
#else
#ifdef THINK_C
	#if __option(a4_globals)
		#define __G_A4__
	#else
		#define __G_A5__
	#endif
#else
	#error unsupported compiler
#endif	/* THINK_C */
#endif	/* __SC__ */
#endif	/* __MWERKS__ */
#endif  /* GENERATINGCFM */



//////////////////////////////////////////////////////////////////////////
//	DEFINE MACROS
//////////////////////////////////////////////////////////////////////////

#ifdef __G_CFM__

	// define new names for cfm
	
	#define PO_SetGlobals(value)
	#define PO_GetGlobals() (0)

	#define PO_EnterGlobals()
	#define PO_ExitGlobals()

	#define PO_PrepareCallback()
	#define PO_EnterCallback()
	#define PO_ExitCallback()

	// define old names for cfm
	
	#define EnterCodeResource()
	#define ExitCodeResource()
	#define PrepareCallback() 
	#define EnterCallback()
	#define ExitCallback()
	
#endif


#ifdef __G_A5__

	// define new names for A5 globals
	
	#define PO_EnterGlobals()
	#define PO_ExitGlobals()
	
	#define PO_GetGlobals()			GetA5()
	#define PO_SetGlobals(value)	SetA5(value)
	#define PO_PrepareCallback()	RememberA5();
	#define PO_EnterCallback()		long oldA5 = SetUpA5();
	#define PO_ExitCallback()		RestoreA5(oldA5);

	// define old names for A5 globals
	
	#define EnterCodeResource()
	#define ExitCodeResource()
	#define PrepareCallback() 		PO_PrepareCallback()
	#define EnterCallback()			PO_EnterCallback()
	#define ExitCallback()			PO_ExitCallback()

#else
#ifdef __G_A4__
	
	// define new names for A4 globals
	// NOTE: old names are defined in SetUpA4.h or A4Stuff.h

	#define PO_GetGlobals()				_GetA4()
	#define PO_SetGlobals(value)		_SetA4(value)
	#define PO_EnterGlobals()			EnterCodeResource()
	#define PO_ExitGlobals()			ExitCodeResource()
	#define PO_PrepareCallback()		RememberA4()
	#define PO_EnterCallback()			long oldA4 = SetUpA4()
	#define PO_ExitCallback()			RestoreA4(oldA4)


#endif /* a4 */
#endif /* a5 */


//////////////////////////////////////////////////////////////////////////
//	DEFINE OR INCLUDE SOME "HELPERS"
//////////////////////////////////////////////////////////////////////////

#ifdef __MWERKS__
	#if __MC68K__
		#include "PO_AsmHelpers.h"
		#ifdef __G_A4__
			#include <a4stuff.h>
			#include <setupa4.h>
		#else
			#include "PO_SetUpA5.h"
		#endif
	#endif
#else
#ifdef THINK_C
	static long _GetA4()
	{
		asm{
			move.l a4,d0
		}
	}
	
	#define _SetA4(value)		asm{move.l value,a4}
	
	static long GetA5()
	{
		asm{
			move.l a5,d0
		}
	}
	
#else
#ifdef __SC__

	#pragma parameter __D0 _GetA4
	static long _GetA4();

	#pragma parameter __D0 GetA5
	static long GetA5();

	#pragma parameter  _SetA4(__A0)
	static void _SetA4(long value);

	static long _GetA4()
	{
		asm(
			0x200C,           //    MOVE.L    A4,D0
			0x4E75			  // 	RTS
		);
		return 0;
	}

	static long GetA5()
	{
		asm(
			0x200D,           //    MOVE.L    A5,D0
			0x4E75			  // 	RTS
		);
		return 0;
	}
	

	static void _SetA4(long value)
	{
		asm(0x2848);    //          MOVE.L    A0,A4
	}
	

#endif	// __SC__
#endif	// THINK_C
#endif	// Metrowerks



#endif

