#pragma once
#ifndef INCLUDED_PO_REF
#define INCLUDED_PO_REF

////////////////////////////////////////////////////////////////////////////////////////
//
//	INTERFACE:		PO_Ref
//
//	DESCRIPTION:	This class implements a "smart pointer" object that will ref
//					count ANY class which is "RefCountable"  See the " Ref Docs "
//					file in this directory for more information.
//				
//	AUTHORS:		Bill Hubauer (BH)
//					Tom Moertel (TM)
//					Geoff Wenger (GW)
//					Brian Zuk	(BZ)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		??			TM		Created
//		??			TM		Revised
//		??			BZ		Stole
//		??			GW		Modified
//		3/14		BH		Re-implemented
//		3/16/97		GW		Added return *this to both operator ='s
//		3/17/97		GW		Added const to IsNullQ, and GetRefCount, and added operator*
//		6/18/97		GW		Added operator!= for PO_Ref<T>.
//		6/18/97		GW		Added operator== and operator!= for T * 
//
////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////
//	PUBLIC INTERFACE
////////////////////////////////////////////////////////////////////////////////////////

template <class T>
class PO_Ref
{
public:
	
	PO_Ref() 
		: _obj(NULL) {}								// default
	PO_Ref(T* inObj) 
		: _obj(NULL) { AddNew(inObj); }				// construct from object
	PO_Ref(const PO_Ref<T>& inRef) 
		: _obj(NULL) { AddNew(inRef._obj); }		// construct from ref
	~PO_Ref();
	
	PO_Ref<T>& operator = (const PO_Ref<T>& inRef)	// assign from another ref
		{ AddNew(inRef._obj); return *this; }
	PO_Ref<T>& operator = (T* inObj)				// assign from an object
		{ AddNew(inObj); return *this; }
	
	T*	operator->() const		{ return _obj; }		// deference
	T&	operator*() const		{ return *_obj; }		// dereference to reference
	
	Boolean	IsNullQ() const		{ return _obj == NULL;}	// test for NULL
	UInt32	GetRefCount() const;
	
	T*		DetachFromReference();	// removes owner ship of this object from
									// all references (See docs before using)
	
	// equal is defined as referencing the same obj
	Boolean operator==(const PO_Ref<T>& inEqualTo) const {return _obj == inEqualTo._obj;}
	Boolean operator!=(const PO_Ref<T>& inEqualTo) const {return _obj != inEqualTo._obj;}
	// or pointing to the same object as our internal pointer
	Boolean operator==(const T* inEqualTo) const {return _obj == inEqualTo;}
	Boolean operator!=(const T* inEqualTo) const {return _obj != inEqualTo;}

private:
	void		AddNew(T* inObj);
	T*	_obj;
};

////////////////////////////////////////////////////////////////////////////////////////
//	IMPLEMENTATION
////////////////////////////////////////////////////////////////////////////////////////

template <class T>
void PO_Ref<T>::AddNew(T* inObj)
{
	if(_obj != NULL){
		_obj->_RemoveReference(true);
	}
	
	_obj = inObj;
	
	if(_obj != NULL){
		_obj->_AddReference();
	}
}

template <class T>
PO_Ref<T>::~PO_Ref()
{
	if(_obj != NULL){
		_obj->_RemoveReference(true);
	}
}

template <class T>
T*	PO_Ref<T>::DetachFromReference()
{
	if(_obj != NULL){
		_obj->_AddReference();
	}
	
	return _obj;
}

template <class T>
UInt32	PO_Ref<T>::GetRefCount() const
{
	if(_obj != NULL){
		return _obj->_GetRefCount();
	}else{
		return 0;
	}
}


////////////////////////////////////////////////////////////////////////////////////////
//	PUBLIC UTILITIES
////////////////////////////////////////////////////////////////////////////////////////

template <class T>
PO_Ref<T>	PO_BeginTempRef(T*	inObj)
{
	PO_Ref<T>	newRef(inObj);
	if(inObj != NULL) inObj->_AddReference();
	return newRef;
}

template <class T>
void	PO_EndTempRef(PO_Ref<T>& inRef)
{
	T*	o = inRef.operator->();
	inRef = NULL;
	if(o != NULL)	o->_RemoveReference(false);
}



// short cuts for naming refs and forward declares
#define DEFINE_REF(oName)	class oName; \
							typedef PO_Ref<oName>	oName##Ref
					
#define DEFINE_TREF(oName)	class oName; \
							template <class T> class PO_TRefCounted; \
							typedef PO_Ref<PO_TRefCounted<oName> >	oName##Ref


#endif
