////////////////////////////////////////////////////////////////
//
//	MODULE:			TR_NetworkTypes
//	
//	DESCRIPTION:	TR_NetworkTypes contains the type primitives
//					and error codes for the TR (Transaction) library
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "TR_NetworkTypes.h"
#include "PO_Assert.h"
#include "TR_InetResolver.h"

////////////////////////////////////////////////////////////////
//	PRIVATE TYPES
////////////////////////////////////////////////////////////////


typedef struct {
	AddrBlock		address;
	EntityName		name;
}AddrName;

enum
{
	tagType = 'TYPE',			// contents OSType
	tagATKName = 'ATK ',		// contents stream
		tagATKObj = 'OBJ ',		// contents text
		tagATKType = 'TYPE',	// contents text
		tagATKZone = 'ZONE',	// contents zone
	tagIPName = 'IPNM',			// contents TIPName
	tagIPAddress = 'IPAD'		// contents TIPAddress
};

struct TIPName
{
	UInt16		port;
	UInt16		pad;
	Str63		name;
};

struct TIPAddress
{
	UInt16		port;
	UInt16		pad;
	UInt32		address;
};

////////////////////////////////////////////////////////////////
//	PRIVATE INTERFACES
////////////////////////////////////////////////////////////////

static void			CalcATKTimeouts(AddrBlock inAddr,UInt16& outTimeoutSeconds,UInt16& outRetries);
static OSErr		NBP_Lookup(EntityName* lookup,short *maxToGet,AddrName** result);
static StringPtr 	NBPGetUserName();



////////////////////////////////////////////////////////////////
//	IMPLEMENTATION
////////////////////////////////////////////////////////////////

OSStatus				TR_NetworkAddress::ConfirmName(const TR_NetworkName& inName)
{
	OSStatus		err = noErr;
	MPPParamBlock	pb;
	char			entity[100];
	
	Str32			name;
	Str32			zone;
	Str32			type;
	
	if(inName.GetATKEntity(name,type,zone)){
	
		NBPSetEntity(entity,name,type,zone);
		pb.NBPinterval = 7;	// recommend values IM:Networking
		pb.NBPcount = 3;
		pb.NBPentityPtr = entity;
		pb.NBPconfirmAddr = atk.fAddr;
		
		err = PConfirmName(&pb,false);
		if(err == noErr){
			
			
		}else if(err == nbpConfDiff){
			atk.fAddr.aSocket = pb.NBPnewSocket;
			err = noErr;
		}else{
			err = trErrorBadName;
		}
	}else{
		err = paramErr;
	}
	
	return err;
}


bool					TR_NetworkAddress::GetATKAddress(AddrBlock& outAddress,
															UInt16& outTimeoutSeconds,UInt16& outRetries) const
{
	if(fType == kAddressTypeAppleTalk){
		outAddress = atk.fAddr;
		outTimeoutSeconds = atk.fTimeout;
		outRetries = atk.fRetries;
		return true;
	}else{
		return false;
	}
}


bool					TR_NetworkAddress::GetIPAddress(UInt32& outHost,UInt16& outPort) const
{
	if(fType == kAddressTypeIP){
		outHost = ip.fHostIP;
		outPort = ip.fHostPort;
		return true;
	}else{
		return false;
	}
}

void					TR_NetworkAddress::InitializeFromIPAddress(UInt32 inHost,UInt16 inPort)
{
	fType = kAddressTypeIP;
	ip.fHostIP = inHost;
	ip.fHostPort = inPort;
	
}


void					TR_NetworkAddress::InitializeFromATK(UInt16 inNet,UInt16 inNode,UInt16 inSocket)
{
	fType = kAddressTypeAppleTalk;
	atk.fAddr.aNet = inNet;
	atk.fAddr.aNode = inNode;
	atk.fAddr.aSocket = inSocket;
	CalcATKTimeouts(atk.fAddr,atk.fTimeout, atk.fRetries);
}



TR_NetworkAddress::TR_NetworkAddress()
	:	fType(kAddressTypeUndefined)
{

}


OSStatus				TR_NetworkAddress::InitializeFromName(const TR_NetworkName& inName)
{
	OSStatus	err = noErr;
	
	if(inName.GetType() == kNameTypeAppleTalk){
		EntityName		name;
		
		bool	isGoodQ = inName.GetATKEntity(name.objStr,name.typeStr,name.zoneStr);
		assert(isGoodQ);
		
		short		maxToGet = 1;
		AddrName	addrName;
		AddrName*	result = &addrName;
		
		err = NBP_Lookup(&name,&maxToGet,&result);
		if(err == noErr){
			if(maxToGet == 1){
				fType = kAddressTypeAppleTalk;
				atk.fAddr = addrName.address;
				CalcATKTimeouts(atk.fAddr,atk.fTimeout,atk.fRetries);			
			}else{
				err = trErrorNameNotFound;
			}
		}else{
			err = trErrorNetworkError;
		}
	
	}else{
		if(inName.GetType() == kNameTypeIPAddress){
			
			inName.GetIPAddress(ip.fHostIP,ip.fHostPort);
			fType = kAddressTypeIP;
			
		
		}else{
			Str255		name;
			UInt16		namePort;
			
			inName.GetIPName(name,namePort);
			name[name[0]+1] = 0;
			err = TR_InetResolver::Resolve((char*)name+1, ip.fHostIP, ip.fHostPort);
			if(err == noErr){
				if(ip.fHostPort == 0){
					ip.fHostPort = namePort;
				}
				fType = kAddressTypeIP;
			}
		}
	}
	
	
	return err;
}


bool				TR_NetworkName::GetATKEntity(Str32 outName,Str32 outType,Str32 outZone) const
{
	if(fType == kNameTypeAppleTalk){
		PLstrcpy(outName,fATK.objStr);
		PLstrcpy(outType,fATK.typeStr);
		PLstrcpy(outZone,fATK.zoneStr);
		return true;
	}else{
		return false;
	}	
}


bool				TR_NetworkName::GetIPAddress(UInt32& outHost,UInt16& outPort) const
{
	if(fType == kNameTypeIPAddress){
		outHost = fIPAddress.fAddress;
		outPort = fIPAddress.fPort;
		return true;
	}else{
		return false;
	}
}

TR_NetworkName::TR_NetworkName(ConstStr32Param inNBPType)
{
	fType = kNameTypeAppleTalk;
	PLstrcpy(fATK.objStr,NBPGetUserName());
	PLstrcpy(fATK.typeStr,inNBPType);
	PLstrcpy(fATK.zoneStr,"\p*");
}



OSStatus			TR_NetworkName::WriteToStream(PO_StreamRef inStream)
{
	(void)inStream;
	DebugStr("\punimplemented: TR_NetworkName::WriteToStream(PO_StreamRef inStream)");
	return paramErr;
}


TR_NetworkName::TR_NetworkName(UInt32 inIPAddress,UInt16 inIPPort)
{
	fType = kNameTypeIPAddress;
	fIPAddress.fPort = inIPPort;
	fIPAddress.fAddress = inIPAddress;
	
}


TR_NetworkName::TR_NetworkName(PO_StreamRef  inConstructFromStream)
{
	(void)inConstructFromStream;
	DebugStr("\punimplemented: TR_NetworkName::TR_NetworkName(PO_StreamRef  inConstructFromStream)");

}


TR_NetworkName::TR_NetworkName(ConstStr63Param inIPNameOrAddress,UInt16 inIPPort)
{
	fType = kNameTypeIPName;
	fIPName.fPort = inIPPort;
	PLstrcpy(fIPName.fName,inIPNameOrAddress);
}


TR_NetworkName::TR_NetworkName(ConstStr32Param inName,ConstStr32Param inType,
						ConstStr32Param inZone)
{
	fType = kNameTypeAppleTalk;
	PLstrcpy(fATK.objStr,inName);
	PLstrcpy(fATK.typeStr,inType);
	PLstrcpy(fATK.zoneStr,inZone);
}

TR_NetworkName::TR_NetworkName()
{
	fType = kNameTypeUndefined;
}


bool				TR_NetworkName::GetIPName(Str255 outName,UInt16& outPort) const
{
	if(fType == kNameTypeIPName){
		PLstrcpy(outName,fIPName.fName);
		outPort = fIPName.fPort;
		return true;
	}else{
		return false;
	}
}





////////////////////////////////////////////////////////////////
//	PRIVATE IMPLEMENTATION
////////////////////////////////////////////////////////////////


static OSErr	NBP_Lookup(EntityName* lookup,short *maxToGet,AddrName** result)
{
	MPPParamBlock		pb;
	char				entity[100];
	char*				buffer = NULL;
	long				bufferSize = (*maxToGet) * 104;
	OSErr				err = noErr;
	
	
	buffer = NewPtr(bufferSize);
	err = MemError();
	if(err == noErr){
		NBPSetEntity(entity,lookup->objStr,lookup->typeStr,lookup->zoneStr);
		pb.NBP.ioCompletion = NULL;
		pb.NBPinterval = 7;
		pb.NBPcount = 3;
		pb.NBPentityPtr = entity;
		pb.NBPretBuffPtr = buffer;
		pb.NBPretBuffSize = bufferSize;
		pb.NBPmaxToGet = *maxToGet;
		
		err = PLookupName(&pb,false);
		if(err == noErr){
			short		index;
			AddrName*	array = *result;
			*maxToGet = pb.NBPnumGotten;
			if(array == NULL){
				array = (AddrName*)NewPtr(sizeof(AddrName)*pb.NBPnumGotten);
				err = MemError();
				*result = array;
			}
			
			if(err == noErr){
				for(index = 1;index<=pb.NBPnumGotten;index++){
					NBPExtract(buffer,pb.NBPnumGotten,index,&array[index-1].name,&array[index-1].address);
				}
			}
		}
	}
		
	if(buffer != NULL){
		DisposePtr(buffer);
	}
			
	return err;
}



static void	CalcATKTimeouts(AddrBlock inAddr,UInt16& outTimeoutSeconds,UInt16& outRetries)
{
	(void)inAddr;
	
	const UInt16		kDefaultTimeoutSeconds = 3;
	const UInt16		kDefaultRetries = 3;

	outTimeoutSeconds = kDefaultTimeoutSeconds;
	outRetries = kDefaultRetries;
}

		


static StringPtr 	NBPGetUserName()
{
	static Str255	userName;
	Handle			h = GetResource('STR ',-16413);
	
	PLstrcpy(userName,"\punknown user");
	if(h != NULL){
		if(**h > 0){
			PLstrcpy(userName,(StringPtr)*h);
		}
	}
	
	return userName;
	
}

