////////////////////////////////////////////////////////////////
//
//	CLASS:			TR_OutgoingRequest_ATK
//	
//	DESCRIPTION:	TR_OutgoingRequest_ATK is the public interface
//					for sending requests to a responder
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "TR_Request_ATK.h"
#include "TR_NetworkTypes.h"
#include "PO_Globals.h"
#include "TR_Buffer.h"
#include "TR_Allocator.h"

////////////////////////////////////////////////////////////////
//	PRIVATE INTERFACES
////////////////////////////////////////////////////////////////

#if GENERATINGCFM
static pascal void	_CompletionProc(ATPParamBlock* inPB);
#else
static pascal void	_CompletionProc();
#endif


////////////////////////////////////////////////////////////////
//	PRIVATE DATA
////////////////////////////////////////////////////////////////

static ATPCompletionUPP		sgCompletionUPP;

struct	__InitTR_Requestor
{
	__InitTR_Requestor();
};

__InitTR_Requestor::__InitTR_Requestor()
{
	sgCompletionUPP = NewATPCompletionProc(_CompletionProc);
}

__InitTR_Requestor	__initTR_Requestor;




////////////////////////////////////////////////////////////////
//	IMPLEMENTATION
////////////////////////////////////////////////////////////////


PO_Ref<TR_Buffer>	TR_OutgoingRequest_ATK::GetResponse()
{
	return fRespBuffer;
}

void				TR_OutgoingRequest_ATK::ATPCompletion()
{
	switch(fPB.ATPioResult){
		case	noErr:
			fResult = noErr;
			break;
		case	reqFailed:
			fResult = trErrorNoResponse;
			break;
		case	tooManyReqs:
		case	noDataArea:
			fResult = trErrorTooManyRequests;
			break;
		case	reqAborted:
			fResult = trErrorRequestCanceled;
			break;
		default:
			fResult = trErrorNetworkError;
			break;
	}
	
	DoNotify();
}


void				TR_OutgoingRequest_ATK::DoNotify()
{
	if(!fSettingNotifyQ){
		if(fNotifyProc != NULL){
			if(fNotifyProcCalled.Grab()){
				(*fNotifyProc)(fNotifyUserData,this);
			}
		}
	}
}


void		TR_OutgoingRequest_ATK::Cancel()
{
	if(fResult == 1){
		ATPParamBlock		pb;
		
		pb.ATPaKillQEl = (Ptr)&fPB;
		PKillSendReq(&pb,false);
	}
}


void			TR_OutgoingRequest_ATK::ATPCompletionStub(ATPParamBlock* inPB)
{
	ATPExtPB*		pb = (ATPExtPB*)inPB;
	long			saveGlobals = PO_GetGlobals();
	PO_SetGlobals(pb->globals);
	
	pb->thisObj->ATPCompletion();
	
	PO_SetGlobals(saveGlobals);
}






void		TR_OutgoingRequest_ATK::WaitUntilComplete()
{
	while(!IsCompleteQ())
		;
}


TR_OutgoingRequest_ATK::TR_OutgoingRequest_ATK
(
	const 	TR_NetworkAddress& 	inAddress,
	PO_Ref<TR_Buffer>			inReqData,
	PO_Ref<TR_Buffer>			inRespBuffer,
	Priority					inPriority 
)
	:	fResult(1),
		fReqData(inReqData),
		fRespBuffer(inRespBuffer),
		fPriority(inPriority),
		fNotifyProc(NULL),
		fSettingNotifyQ(false)
{
	AddrBlock		addr;
	UInt16			timeout;
	UInt16			retries;
	OSStatus		err = noErr;
	
	if(inReqData->GetLogicalSize() > TR_RequestFactory::MaxRequestSizeForAddressType(inAddress.GetType())){
		err = trErrorRequestTooBig;
	}else{
		
		if(inAddress.GetATKAddress(addr, timeout, retries)){
			
			fPB.ATPioCompletion = sgCompletionUPP;
			fPB.ATPatpFlags = atpXOvalue | atpSendChkvalue;
			fPB.ATPaddrBlock = addr;
			fPB.ATPreqLength = inReqData->GetLogicalSize();
			fPB.ATPreqPointer = (Ptr)inReqData->GetBuffer();
			fPB.ATPnumOfBuffs = BuildBDS((Ptr)inRespBuffer->GetBuffer(),
										(char*)fBDS,inRespBuffer->GetPhysicalSize());
			fPB.ATPbdsPointer = (char*)fBDS;
			fPB.ATPtimeOutVal = timeout;
			fPB.ATPretryCount = retries;
			fPB.globals = PO_GetGlobals();
			fPB.thisObj = this;
			
			PSendRequest(&fPB,true);
			
		}else{
			DebugStr("\pIP Unimplemented");
			err = paramErr;
		}
	}
	
	if(err != noErr){
		fResult = err;
	}
}

void	TR_OutgoingRequest_ATK::operator delete(void* inObj)//Override

{
	if(inObj != NULL){
		TR_OutgoingRequest_ATK*	thisObj = (TR_OutgoingRequest_ATK*)inObj;
		thisObj->fAllocator->Free(inObj);
	}
}


void*	TR_OutgoingRequest_ATK::operator new(UInt32 inSize,TR_Allocator* inAllocator)//Override

{
	if(inAllocator == NULL){
		inAllocator = gDefaultAllocator;
	}
	
	TR_OutgoingRequest_ATK*		newObj = (TR_OutgoingRequest_ATK*)inAllocator->Allocate(inSize);
	if(newObj != NULL){
		newObj->fAllocator = inAllocator;
	}
	
	return newObj;
}



void		TR_OutgoingRequest_ATK::NotifyWhenComplete(NotifyProc inProc,void* inUserData)
{
	fSettingNotifyQ = true;
	
	fNotifyProc = inProc;
	fNotifyUserData = inUserData;

	fSettingNotifyQ = false;
	
	if(fResult != 1){
		DoNotify();
	}
}


Boolean		TR_OutgoingRequest_ATK::IsCompleteQ()
{
	return (fResult != 1);
}


OSStatus			TR_OutgoingRequest_ATK::GetResult()
{
	return fResult;
}


////////////////////////////////////////////////////////////////
//	PRIVATE IMPLEMENTATION
////////////////////////////////////////////////////////////////


#if GENERATINGCFM

	static pascal void	_CompletionProc(ATPParamBlock* inPB)
	{
		TR_OutgoingRequest_ATK::ATPCompletionStub(inPB);
	}
	
#else

	static pascal void	_CompletionProc()
	{
		ATPParamBlock*		inPB = (ATPParamBlock*) GetA0();
		TR_OutgoingRequest_ATK::ATPCompletionStub(inPB);
	}
#endif
