#pragma once
#ifndef INCLUDED_TR_RESPONDER
#define INCLUDED_TR_RESPONDER
////////////////////////////////////////////////////////////////
//
//	CLASS:			TR_Responder
//	
//	DESCRIPTION:	TR_Responder is the public interface
//					for receiving and responding to requests
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "PO_Ref.h"
#include "TR_NetworkTypes.h"
#include "PO_TQueue.h"


////////////////////////////////////////////////////////////////
//	REFERENCED BY NAME
////////////////////////////////////////////////////////////////

class	TR_ResponderEventReceiver;
class	TR_IncommingRequest_Common;
class	TR_IncommingRequest;
class	TR_Responder_Common;
class	TR_Allocator;
class	TR_Buffer;
class 	TR_Responder;

////////////////////////////////////////////////////////////////
//	PUBLIC TYPES AND CLASSES
////////////////////////////////////////////////////////////////

typedef UInt32		TR_RequestCookie;
const	UInt32		kInvalidRequestCookie = 0;





#pragma mark class TR_ResponderFactory
struct TR_ResponderFactory
{
	static OSStatus	New
	(
		const TR_NetworkName&	inName,						// network name of this responder
		UInt32 					inInitialNumRequests,		// initial size of request pool*
		UInt32					inMinFreeRequests,			// if request pool dips below this, the pool is grown
		UInt32					inGrowSize,					// grow increment if minFreeRequests if violated
		UInt32					inMaxNumberRequests,		// max size pool will grow to
		
		UInt32					inMaxRequestBufferSize,		// request buffer size
		TR_Allocator*			inRequestAllocator,			// allocator used to allocate buffers and request pool data
															// NOTE: allocate is only done at memory safe time
															//       you can pass NULL for default allocation
		TR_Responder*&			outResponder				// new responder, call delete to when done with it.
	);
	
	//	* Request Pool:  The size of the request pool is the maximum number
	//					 of simultanious requests that can be handled.
 
};







#pragma mark class TR_Responder
class	TR_Responder
{
	public:		
		
		virtual OSStatus	Initialize
		(
			const TR_NetworkName&			 	inName,
			UInt32 								inInitialNumRequests,
			UInt32								inMinFreeRequests,
			UInt32								inGrowSize,
			UInt32								inMaxNumRequests,
			UInt32								inMaxRequestBufferSize,
			TR_Allocator*						inRequestAllocator 
		) = 0;
		
		// query responder
		
		virtual void	GetNetworkAddress(TR_NetworkAddress& outMyAddress) = 0;
		
		// request notification
		
		virtual void	AttachEventReceiver(TR_ResponderEventReceiver* inEventReceiver) = 0;
		virtual void	RemoveEventReceiver(TR_ResponderEventReceiver* inEventReceiver) = 0;

		// query a request new request
		
		virtual bool	QueryRequestData(TR_RequestCookie inCookie,PO_Ref<TR_Buffer>& outRequestData) = 0;
		virtual bool	QueryRequestSender(TR_RequestCookie inCookie,TR_NetworkAddress& outSender) = 0;
		
		// commit to handling a new request
		
		virtual TR_IncommingRequest*	CommitRequest(TR_RequestCookie inCookie) = 0;	

		// give idle time for request pool maintainence
		
		virtual void	Idle() = 0;
		
		// query the request pool
		
		virtual UInt32	GetPoolSize() = 0;
		virtual UInt32	GetNumFree() = 0;
		
		// construct and destruct
		
		TR_Responder();
		virtual ~TR_Responder();
};










// Protocol class for receiving events

#pragma mark class TR_ResponderEventReceiver
class	TR_ResponderEventReceiver : public PO_TQueueElem<TR_ResponderEventReceiver>
{
public:
			TR_ResponderEventReceiver();
	virtual ~TR_ResponderEventReceiver();

protected:
	friend class	TR_Responder_Common;
	
	virtual	void	NotifyNewRequest(TR_Responder* inResponder,TR_RequestCookie inRequest) = 0;
	virtual void	NotifyResponseComplete(TR_Responder* inResponder,TR_IncommingRequest* inRequest,
											void* inUserData) = 0;

private:
	void			AttachTo(TR_Responder* inResponder);
	
	
	TR_Responder_Common*	fResponder;	
};











// Iterator for getting all the incomeing requests
#pragma mark class TR_RequestIterator
class	TR_RequestIterator
{
	public:
		
		TR_RequestIterator(TR_Responder& inResponder);
		
		bool				MoreQ();
		TR_RequestCookie	GetNext();
		void				Reset();
		
		
	private:
		PO_TQIterator<TR_IncommingRequest_Common>		fIter;
};








// IncommingRequest object.

#pragma mark class TR_IncommingRequest
class	TR_IncommingRequest	: public PO_TQueueElem<TR_IncommingRequest>
{
	public:
	
		// accessors
		
		virtual PO_Ref<TR_Buffer>	GetRequestBuffer() const = 0;
		virtual void				GetSender(TR_NetworkAddress& outSender) const = 0;
		
		
		// manipulators
		
		virtual void				CancelRequest() = 0;
		virtual void				SendResponse(PO_Ref<TR_Buffer> inResponseBuffer,
												void* inUserData, bool inAsyncQ) = 0;
 
	

	protected:
		friend class TR_Responder_Common;
		TR_IncommingRequest();
		virtual ~TR_IncommingRequest();
};




#endif