#include <UGraphicUtils.h>

#include "CImageView.h"
static LGWorld* MakeGWorld(const Rect& bounds, int bitDepth);

// ---------------------------------------------------------------------------
//		 CImageView(LStream*)
// ---------------------------------------------------------------------------
//	Construct a new Button from the data in a Stream

CImageView::CImageView (LStream * inStream)
	: LView(inStream), fImage(0), fOwnsImage(false), fMask(0), fOwnsMask(false)
{
}

// ---------------------------------------------------------------------------
//		 ~CImageView()
// ---------------------------------------------------------------------------
//	Destroy CImageView object.

CImageView::~CImageView ()
{
	if (fOwnsImage)
		delete fImage;
	if (fOwnsMask)
		delete fMask;
}

void 
CImageView::AssignImage (LGWorld *image, bool ownsImage)
{
	LGWorld *oldImage = fImage;
	fImage = 0;
	if (fOwnsImage)
		delete oldImage;
	fImage = image;
	fOwnsImage = ownsImage;
	if (fImage) {
		Rect r;
		fImage->GetBounds(r);
		int imageWidth = UGraphicUtils::RectWidth(r);
		int imageHeight = UGraphicUtils::RectHeight(r);
		SDimension16 frameSize;
		GetFrameSize(frameSize);
		fOffsetH = (frameSize.width - imageWidth)/2;
		fOffsetV = (frameSize.height - imageHeight)/2;
	}
	Refresh();
}

void 
CImageView::AssignMask (LGWorld *image, bool ownsMask)
{
	LGWorld *oldMask = fMask;
	fMask = 0;
	if (fOwnsMask)
		delete oldMask;
	fMask = image;
	fOwnsMask = ownsMask;
	Refresh();
}

void 
CImageView::AdoptImage (LGWorld *image)
{
	AssignImage(image, true);
}

void 
CImageView::SetImage (LGWorld *image)
{
	AssignImage(image, false);
}

void 
CImageView::AdoptMask (LGWorld *image)
{
	AssignMask(image, true);
}

void 
CImageView::SetMask (LGWorld *image)
{
	AssignMask(image, false);
}

inline BitMap*
BitMapFromGWorld(LGWorld* gWorld)
{
	return &((GrafPtr)gWorld->GetMacGWorld())->portBits;
}

inline BitMap*
BitMapFromGrafPtr(GrafPtr grafPtr)
{
	return &(grafPtr->portBits);
}

void
CImageView::DrawWithMask(LGWorld* image, LGWorld* mask, GrafPtr targetPort, int offsetH, int offsetV)
{
	StColorState state;
	state.Normalize();
	// Set ForeColor to black, and BackColor to white
	// NOTE:  This is necessary to prevent copybits from displaying
	// unwanted colors  in the copied image.
	Rect bounds, target;
	image->GetBounds(bounds);
	target = bounds;
	::OffsetRect(&target, offsetH, offsetV);
	CopyMask (BitMapFromGWorld(image),
			  BitMapFromGWorld(mask),
			  BitMapFromGrafPtr(targetPort),
			  &bounds, &bounds, &target);
}

void
CImageView::DrawSelf()
{
	if (fImage) {
		if (fMask) {
			DrawWithMask(fImage, fMask, GetMacPort(), fOffsetH, fOffsetV);
		} else {
			Rect r;
			fImage->GetBounds(r);
			::OffsetRect(&r, fOffsetH, fOffsetV);
			fImage->CopyImage(GetMacPort(), r);
		}
	}
}

static LGWorld*
MakeGWorld(const Rect& bounds, int bitDepth)
{
	LGWorld *world = new LGWorld(bounds, bitDepth, 0);
	if (world == 0)
		world = new LGWorld(bounds, bitDepth, useTempMem);
	
	return world;
}

LGWorld*
CImageView::GWorldFromPict(int pictResID, int bitDepth)
{
	PicHandle h = GetPicture(pictResID);
	ThrowIfResFail_(h);
	Rect bounds = (**h).picFrame;
	
	StDeleter<LGWorld> world(MakeGWorld(bounds, bitDepth));
	
	LGWorld *worldP = world.Get();
	ThrowIfNil_(worldP);
	
	worldP->BeginDrawing();
	DrawPicture(h, &bounds);
	worldP->EndDrawing();
	
	ReleaseResource((Handle)h);
	return world.Release();
}

static void GetTemporaryFile(FSSpec *outSpec);
static void GetTemporaryFileOnVolume(short vRefNum, FSSpec *outSpec);

static void GetTemporaryFileOnVolume(short vRefNum, FSSpec *outSpec)
{
	short foundVRefNum;
	long foundDirID;
	OSErr err = ::FindFolder(vRefNum, kChewableItemsFolderType, true, 
				&foundVRefNum, &foundDirID);
	if (err != noErr)
		ThrowIfOSErr_(::FindFolder(vRefNum, kTemporaryFolderType, true, 
					&foundVRefNum, &foundDirID));
				
	long fileName = Random();
	do {
		LStr255 tempName("\pixt-");
		LStr255 number(fileName);
		tempName.Append(number);
		tempName.Append("\p.gif");
		fileName += 1;
		err = ::FSMakeFSSpec(foundVRefNum, foundDirID, tempName, outSpec);
	} while (err == noErr);
	if (err != fnfErr)
		Throw_(err);
}

static void GetTemporaryFile(FSSpec *outSpec)
{
	GetTemporaryFileOnVolume(kOnSystemDisk, outSpec);
}

LGWorld*
CImageView::GWorldFromGIF(void *p, int length, int bitDepth)
{
	LGWorld *world = 0;
	
	Try_{
		FSSpec spec;
		GetTemporaryFile(&spec);
		LFileStream aFile(spec);
		aFile.CreateNewDataFile('GIFf', 0);
		aFile.OpenDataFork(fsRdWrPerm);
		aFile.WriteBlock(p, length);
		aFile.CloseDataFork();
		ComponentInstance gi;
		ThrowIfOSErr_(GetGraphicsImporterForFile(&spec, &gi));
		Rect r;
		ThrowIfOSErr_(GraphicsImportGetNaturalBounds(gi, &r));
		world = MakeGWorld(r, bitDepth);
		ThrowIfMemFail_(world);
		ThrowIfOSErr_(GraphicsImportSetGWorld(gi, world->GetMacGWorld(), 0));
		ThrowIfOSErr_(GraphicsImportDraw(gi));
		ThrowIfOSErr_(GraphicsImportSetGWorld(gi, 0, 0));
		ThrowIfOSErr_(CloseComponent(gi));
		FSpDelete(&spec);
	} Catch_ (err) {
	}
	return world;
}