// ===========================================================================
//	CPalmPadApp.cp 			1994-1998 Metrowerks Inc. All rights reserved.
// ===========================================================================
//	This file contains the starter code for a PalmPad PowerPlant project that
//  includes precompiled headers and both debug and release targets.

#include "CPalmPadApp.h"
#include "IRTransfer.h"
#include "CImageView.h"
#include "CCDDrive.h"

#include <LGrowZone.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <URegistrar.h>
#include <UControlRegistry.h>
#include <LStaticText.h>
#include <QDOffscreen.h>
#include <PictUtils.h>


template <class T>
void FindPane (LView *view, const PaneIDT& paneID, T** result)
{
	*result = 0;
	T* temp = dynamic_cast<T*>(view->FindPaneByID(paneID));
	ThrowIfNil_(temp);
	*result = temp;
}

const PP_PowerPlant::ResIDT	wind_SampleWindow = 128;	// EXAMPLE, create a new window
const PP_PowerPlant::ResIDT wind_ImageWindow = 129;

class CMouseDataTransfer;
static LPeriodical *gIdler = 0;
static bool gSending = true;
static CIRTransfer* gIR = 0;

static StringPtr
gClosedOpen[] = 
{
	"\pclosed", "\popen"
};

static StringPtr
gIRStatus[] = 
{
	"\pestablished", "\pnot established", "\pbusy"
};

class CMouseDataTransfer : public LPeriodical {
public:
	CMouseDataTransfer(LWindow* target);
	virtual ~CMouseDataTransfer();

	virtual	void	SpendTime(
							const EventRecord&		inMacEvent);

private:
	LStaticText*	fCursor;
	LStaticText*	fInputOpen;
	LStaticText*	fOutputOpen;
	LStaticText*	fOutputStatus;
	LStaticText*	fLastPacket;
	Point			fLastPt;
};

CMouseDataTransfer::CMouseDataTransfer(LWindow* target)
{
	fLastPt.h = fLastPt.v = 0;
	FindPane(target, 'mloc', &fCursor);
	FindPane(target, 'irin', &fInputOpen);
	FindPane(target, 'irou', &fOutputOpen);
	FindPane(target, 'irst', &fOutputStatus);
	FindPane(target, 'last', &fLastPacket);
}

CMouseDataTransfer::~CMouseDataTransfer()
{
	if (gIdler == this)
		gIdler = 0;
}

static bool operator!= (const Point& a, const Point& b)
{
	return !EqualPt(a, b);
}

void
CMouseDataTransfer::SpendTime(const EventRecord& theEvent)
{
	Point pt = theEvent.where;
	if (pt != fLastPt) {
		fCursor->SetDescriptor(LStr255(pt.h) + "\p, " + LStr255(pt.v));
		fLastPt = pt;
	}
	fInputOpen->SetDescriptor(gClosedOpen[gIR->InOpen()]);
	fOutputOpen->SetDescriptor(gClosedOpen[gIR->OutOpen()]);
	fOutputStatus->SetDescriptor(gIRStatus[gIR->OutStatus()]);
	fLastPacket->SetValue(gIR->LastPacketTick());
}

// ===========================================================================
//		 Main Program
// ===========================================================================

int main()
{
#ifndef NDEBUG									
	SetDebugThrow_(PP_PowerPlant::debugAction_Alert);	// Set Debugging options
	SetDebugSignal_(PP_PowerPlant::debugAction_Alert);
#endif

	PP_PowerPlant::InitializeHeap(3);					// Initialize Memory Manager
														// Parameter is number of Master Pointer
														// blocks to allocate
	
	PP_PowerPlant::UQDGlobals::InitializeToolbox(&qd);	// Initialize standard Toolbox managers
	
	new PP_PowerPlant::LGrowZone(20000);				// Install a GrowZone function to catch
														// low memory situations.

	CPalmPadApp	theApp;									// create instance of your application
	theApp.Run();
	
	return 0;
}


// ---------------------------------------------------------------------------
//		 CPalmPadApp
// ---------------------------------------------------------------------------
//	Constructor

CPalmPadApp::CPalmPadApp()
{
	RegisterClass_(LWindow);
	RegisterClass_(CImageView);
	RegisterClass_(LPicture);
	RegisterClass_(LDialogBox);
	UControlRegistry::RegisterClasses();
	RgnHandle h(::GetGrayRgn());
	fScreenRect = (**h).rgnBBox;
	fCursorDevice = 0;
	ThrowIfOSErr_(CursorDeviceNextDevice(&fCursorDevice));
	gIR = new CIRTransfer();
	gIR->OpenIR();
	
	new UMainThread();
	AddAttachment(new LYieldAttachment());
	
	LSimpleThread *thread = new LSimpleThread(CPalmPadApp::ReadDataWrapper, this);
	thread->Resume();
}


// ---------------------------------------------------------------------------
//		 ~CPalmPadApp
// ---------------------------------------------------------------------------
//	Destructor

CPalmPadApp::~CPalmPadApp()
{
}

// ---------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------
//	This method lets you do something when the application starts up
//	without a document. For example, you could issue your own new command.

void
CPalmPadApp::StartUp()
{
	ObeyCommand(PP_PowerPlant::cmd_New, nil);			// EXAMPLE, create a new window
}

// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	This method lets the application respond to commands like Menu commands

Boolean
CPalmPadApp::ObeyCommand(
	PP_PowerPlant::CommandT	inCommand,
	void					*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) {
		case cmd_About:
			DoAbout();
			break;
			
		case 'Send':
			DoEject();
			break;
			
		case 'List':
			break;
			
		case 'ClIR':
			CloseDriver(0x5e);
			CloseDriver(0x5f);
			break;
			
		case 'imgv':
			CaptureScreen();
			break;

		case PP_PowerPlant::cmd_New:
										
			PP_PowerPlant::LWindow	*theWindow = 
									PP_PowerPlant::LWindow::CreateWindow(wind_SampleWindow, this);
			ThrowIfNil_(theWindow);
			
			// LWindow is not initially visible in PPob resource
			CMouseDataTransfer* tracker = new CMouseDataTransfer(theWindow);
			tracker->StartIdling();
			theWindow->Show();
			break;

		// Any that you don't handle, such as cmd_About and cmd_Quit,
		// will be passed up to LApplication
		default:
			cmdHandled = PP_PowerPlant::LApplication::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	This method enables menu items.

void
CPalmPadApp::FindCommandStatus(
	PP_PowerPlant::CommandT	inCommand,
	Boolean					&outEnabled,
	Boolean					&outUsesMark,
	PP_PowerPlant::Char16	&outMark,
	Str255					outName)
{

	switch (inCommand) {
	
		// Return menu item status according to command messages.
		case PP_PowerPlant::cmd_New:
			outEnabled = true;
			break;

		case 'Send':
		case 'List':
			outEnabled = true;
			break;

		case 'ClIR':
			outEnabled = true;
			break;

		case 'imgv':
			outEnabled = true;
			break;

		// Any that you don't handle, such as cmd_About and cmd_Quit,
		// will be passed up to LApplication
		default:
			PP_PowerPlant::LApplication::FindCommandStatus(inCommand, outEnabled,
												outUsesMark, outMark, outName);
			break;
	}
}

void CPalmPadApp::HandlePacket()
{
	const int kBufferSize = 32*1024;
	static Uint8 buffer[kBufferSize];
	Int32 dataSize = kBufferSize;
	int packetType;
	if (gIR->ReceivePacket(packetType, buffer, dataSize)) {
		if (packetType == 1)
			DoEject();
	}
}

void
CPalmPadApp::HandleMousePacket(const void *bufferP, Int32)
{
	IRTransfer::MouseData* mouseDataP = (IRTransfer::MouseData*)bufferP;
	CursorDeviceMove(fCursorDevice, mouseDataP->currX, mouseDataP->currY);
}

void
CPalmPadApp::HandleButtonsPacket(const void *bufferP, Int32)
{
	IRTransfer::Buttons* buttonsDataP = (IRTransfer::Buttons*)bufferP;
	CursorDeviceButtons(fCursorDevice, buttonsDataP->buttons);
}

void
CPalmPadApp::HandleRequestGraphicsPacket(const void *, Int32)
{
	StDeleter<LGWorld> newGWorld(DumpScreenArea());
	StPointerBlock grBlock((Ptr)GWorldToGraphicsData(newGWorld.Get()));
	IRTransfer::GraphicsData* gr = (IRTransfer::GraphicsData*)grBlock.Get();
	gIR->SendPacket(*gr);
}

pascal void CloseIR(void);
pascal void CloseIR(void)
{
	delete gIR;
}

void
CPalmPadApp::ReadDataWrapper(LThread&, void* arg)
{
	((CPalmPadApp*)arg)->ReadData();
}

void
CPalmPadApp::ReadData()
{
	while (1) {
		HandlePacket();
		LThread::Yield();
	}
}

void
CPalmPadApp::SendDataWrapper(LThread&, void* arg)
{
	((CPalmPadApp*)arg)->SendData();
}

void
CPalmPadApp::SendData()
{
}

// dump an area of the screen
// return a PicHandle or nil
LGWorld* CPalmPadApp::DumpScreenArea()
{
	CGrafPtr	wPort ;
	CGrafPtr	savedPort ;
	GDHandle	oldDevice ;
	PicHandle	thePict = nil ;
	
	GWorldPtr	theNewWorld ;
	GDHandle	gDevice;
	
	Rect		gDeviceRect;
	Rect		intersectingRect;
	Rect 		area ;
	
	RgnHandle	grayRgn = GetGrayRgn() ;
	Rect		wideOpenRect = (**grayRgn).rgnBBox ;

	// save our world
	GetGWorld( &savedPort, &oldDevice ) ;
		
	// get the window managers grafport
	GetCWMgrPort( &wPort ) ;

	// and make the window managers grafport our current port
	SetPort( (GrafPtr)wPort ) ;

	ClipRect( &wideOpenRect ) ;
	
	Point mousePt;
	GetMouse(&mousePt);
	LocalToGlobal(&mousePt);
	SetRect(&area, mousePt.h-80, mousePt.v-80, mousePt.h+80, mousePt.v+80);
	Rect bounds;
	SetRect(&bounds, 0, 0, 160, 160);

	LGWorld *newWorld = new LGWorld(bounds, 1);
	theNewWorld = newWorld->GetMacGWorld();
	
	SetGWorld( (CGrafPtr)theNewWorld, nil ) ;

	// Get the handle to the first device in the global device list.
	// loop through the device list, checking if the rect defined by area 
	// intersects the device
	
	for( gDevice = GetDeviceList(); gDevice != nil; gDevice = GetNextDevice( gDevice )) {
		
		// Get the device's gdRect and convert it to local coordinates.
		gDeviceRect = (**gDevice).gdRect;
			
		GlobalToLocal( &topLeft( gDeviceRect ) );
		GlobalToLocal( &botRight( gDeviceRect ) );
		
		// Check if the app's window rect intersects the device's, and if it
		// does, set the clip region's rect to the intersection and copy into
		// our offscreen buffer
		
		if (SectRect( &area, &gDeviceRect, &intersectingRect ))
		{
			Rect r = intersectingRect;
			OffsetRect(&r, -area.left, -area.top);
			//ClipRect( &intersectingRect );
			CopyBits( (BitMap *)(*(**gDevice).gdPMap), 
					  (BitMap *)(*(*theNewWorld).portPixMap), 
					  &intersectingRect, 
					  &r, 
					  srcCopy, 
					  nil );
		}
	}
	SetGWorld( savedPort, oldDevice ) ;
	
	return newWorld;
}

void
CPalmPadApp::CaptureScreen()
{
	// do the capture
	LGWorld *newGWorld = DumpScreenArea();
	
	PP_PowerPlant::LWindow	*theWindow = 
							PP_PowerPlant::LWindow::CreateWindow(wind_ImageWindow, this);
	
	ThrowIfNil_(theWindow);
	CImageView* iView;
	FindPane(theWindow, 'imgv', &iView);
	StPointerBlock grBlock((Ptr)GWorldToGraphicsData(newGWorld));
	IRTransfer::GraphicsData* gr = (IRTransfer::GraphicsData*)grBlock.Get();
	gIR->SendPacket(*gr);
	iView->AdoptImage(newGWorld);
	theWindow->Show();
}

IRTransfer::GraphicsData*
CPalmPadApp::GWorldToGraphicsData(LGWorld* world)
{
	GWorldPtr theWorld = world->GetMacGWorld();
	PixMapHandle pmh = GetGWorldPixMap(theWorld);
	long rowBytes = (**pmh).rowBytes & 0x3fff;
	Rect bounds = (**pmh).bounds;
	long imageHeight = bounds.bottom - bounds.top;
	int imageSize = imageHeight * rowBytes;
	StPointerBlock graphics(IRTransfer::GraphicsDataHeaderSize + imageSize);
	IRTransfer::GraphicsData* gPtr = (IRTransfer::GraphicsData*)graphics.Get();
	gPtr->rowBytes = rowBytes;
	gPtr->height = imageHeight;
	gPtr->pixelDepth = (**pmh).pixelSize;
	gPtr->width = bounds.right - bounds.left;
	Boolean wasLocked = LockPixels(pmh);
	Uint8 *p = (Uint8*)(**pmh).baseAddr;
	Uint8 *d = &gPtr->data[0];
	for (int i = 0; i < imageHeight; ++i) {
		for (int j = 0; j < rowBytes; j++) {
			*d++ = *p++;
		}
	}
	
	return (IRTransfer::GraphicsData*)graphics.Release();
}

void
CPalmPadApp::SendNullPacket()
{
	char c;
	gIR->SendPacket(0, &c, 0);
}

void
CPalmPadApp::DoEject()
{
	CDDrive::Iterator iter;
	while (iter.More())
		iter.Next().Eject();
}

void
CPalmPadApp::DoAbout()
{
	StDialogHandler handler(130, this);
	CommandT msg;
	do {
		msg = handler.DoDialog();;
	} while(msg != msg_OK);
}
