#include "IRTransfer.h"

#include <serial.h>

#undef USE_SERIAL
#ifdef USE_SERIAL
ConstStringPtr kIRCommIn = "\p.AIn";
ConstStringPtr kIRCommOut = "\p.AOut";
#else
ConstStringPtr kIRCommIn = "\p.IrIn";
ConstStringPtr kIRCommOut = "\p.IrOut";
#endif

CIRTransfer::CIRTransfer()
	: fOpened(false), fInRefNum(0), fOutRefNum(0), fCantOpen(false), fLastPacketTick(0)
{
}

CIRTransfer::~CIRTransfer()
{
	CloseIR();
}

bool
CIRTransfer::OpenIR()
{
	if (fCantOpen)
		return false;
		
	fCantOpen = true;
	ThrowIfOSErr_(OpenDriver(kIRCommIn, &fInRefNum));
	ThrowIfOSErr_(OpenDriver(kIRCommOut, &fOutRefNum));
#ifdef USE_SERIAL
	SerReset(fInRefNum, baud14400+stop10+noParity+data8);
	SerReset(fOutRefNum, baud14400+stop10+noParity+data8);
#endif
	fOpened = true;
	fCantOpen = false;
	return true;
}

void
CIRTransfer::CloseIR()
{
	if (fInRefNum)
		CloseDriver(fInRefNum);
	if (fOutRefNum)
		CloseDriver(fOutRefNum);
}

void
CIRTransfer::SendPacket(const IRTransfer::MouseData& mouseData)
{
	SendPacket(mousePacket, &mouseData, sizeof(mouseData));
}

void
CIRTransfer::SendPacket(const IRTransfer::GraphicsData& grData)
{
	int imageSize = grData.height * grData.rowBytes;
	SendPacket(graphicsPacket, &grData, IRTransfer::GraphicsDataHeaderSize + imageSize);
}

void
CIRTransfer::SendPacket(int packetType, const void *packetData, Int32 packetLength)
{
	if (fCantOpen)
		return;

	if (!fOpened)
		OpenIR();
	
	Int32 packetSizeWithHeader = packetLength + sizeof(IRTransfer::PacketHeader);
	StPointerBlock packetBuffer(packetSizeWithHeader);
	Ptr p(packetBuffer.Get()); 
	::BlockMoveData(&packetType, p, sizeof(packetType));
	p += sizeof(packetType);
	::BlockMoveData(&packetLength, p, sizeof(packetLength));
	p += sizeof(packetLength);
	::BlockMoveData(packetData, p, packetLength);
	ThrowIfOSErr_(FSWrite(fOutRefNum, &packetSizeWithHeader, packetBuffer.Get()));
}

bool
CIRTransfer::ReceivePacket(int& packetType, void *packetData, Int32& packetLength)
{
	if (!fOpened)
		OpenIR();
		
	long bytesAvail;
	IRTransfer::PacketHeader header;
	do {
		ThrowIfOSErr_(SerGetBuf(fInRefNum, &bytesAvail));
		LThread::Yield();
	} while (bytesAvail < sizeof(header));
	
	long dataRead = sizeof(header);
	OSErr err = FSRead(fInRefNum, &dataRead, &header);
	if (err == noErr && dataRead  == sizeof(header)) {
		long dataRead = header.size;
		if (dataRead > 0) {
			do {
				ThrowIfOSErr_(SerGetBuf(fInRefNum, &bytesAvail));
				LThread::Yield();
			} while (bytesAvail < dataRead);
			err = FSRead(fInRefNum, &dataRead, packetData);
		}
		packetLength = dataRead;
		packetType = header.type;
		fLastPacketTick = TickCount();
		return true;
	}
	return false;
}

int
CIRTransfer::OutStatus() const
{
	SerStaRec rec;
	OSErr err = SerStatus(fOutRefNum, &rec);
	if (err != noErr)
		return err;
	return rec.ctsHold;
}
