#include "Utilities.h"
#include "Stringiness.h"
#include "Str255.h"
#include "MoreFilesExtras.h"
#include "LErrorMsg.h"
#include "ErrorStrings.h"
#include "ErrorCodes.h"
#include "ConverterErrorStrings.h"
#include "CFileMaker.h"
#ifdef CONDUIT
#include "SyncMgr.h"
#include "UDebugOut.h"
#endif

void
ClearBlock(void *p, int length)
{
	char *ptr = (char *)p;
	char *limitPtr = ptr + length;
	
	while (ptr < limitPtr)
		*ptr++ = 0;
}

void GetPreferencesFolder(FSSpec& spec)
{
	ThrowIfOSErr_(::FindFolder(kOnSystemDisk, kPreferencesFolderType, true, 
				&spec.vRefNum, &spec.parID));
}

void GetTemporaryFileOnVolume(short vRefNum, FSSpec *outSpec)
{
	short foundVRefNum;
	long foundDirID;
	OSErr err = ::FindFolder(vRefNum, kChewableItemsFolderType, true, 
				&foundVRefNum, &foundDirID);
	if (err != noErr)
		ThrowIfOSErr_(::FindFolder(vRefNum, kTemporaryFolderType, true, 
					&foundVRefNum, &foundDirID));
				
	long fileName = Random();
	do {
		LStr255 tempName("\pixt-");
		LStr255 number(fileName);
		tempName.Append(number);
		fileName += 1;
		err = ::FSMakeFSSpec(foundVRefNum, foundDirID, tempName, outSpec);
	} while (err == noErr);
	if (err != fnfErr)
		Throw_(err);
}

void GetTemporaryFile(FSSpec *outSpec)
{
	GetTemporaryFileOnVolume(kOnSystemDisk, outSpec);
}

void
GetTemporaryFileNear(const FSSpec& inSpec, FSSpec *outSpec)
{
	GetTemporaryFileOnVolume(inSpec.vRefNum, outSpec);
}

bool
GetTempFileNamed(ConstStringPtr fileName, FSSpec *outSpec)
{
	short foundVRefNum;
	long foundDirID;
	OSErr err = ::FindFolder(kOnSystemDisk, kChewableItemsFolderType, true, 
				&foundVRefNum, &foundDirID);
	if (err != noErr)
		ThrowIfOSErr_(::FindFolder(kOnSystemDisk, kTemporaryFolderType, true, 
					&foundVRefNum, &foundDirID));
	err = ::FSMakeFSSpec(foundVRefNum, foundDirID, fileName, outSpec);
	if (err != noErr && err != fnfErr)
		Throw_(err);
	return err == noErr;
}

CFSSpec::CFSSpec()
{
	fSpec.name[0] = 0;
	fSpec.vRefNum = 0;
	fSpec.parID = 0;
}

CFSSpec::CFSSpec(const FSSpec& spec)
	: fSpec(spec)
{
}

Ptr LockGWorldPixels(LGWorld* world)
{
	GWorldPtr gworld = world->GetMacGWorld();
	PixMapHandle aPixMap = GetGWorldPixMap(gworld);
	ThrowIfNot_(LockPixels(aPixMap));
	return GetPixBaseAddr(aPixMap);
}

void UnlockGWorldPixels(LGWorld* world)
{
	GWorldPtr gworld = world->GetMacGWorld();
	PixMapHandle aPixMap = GetGWorldPixMap(gworld);
	UnlockPixels(aPixMap);
}

int GWorldRowBytes(LGWorld* world)
{
	GWorldPtr gworld = world->GetMacGWorld();
	PixMapHandle aPixMap = GetGWorldPixMap(gworld);
	return (**aPixMap).rowBytes & 0x7fff;
}

void Normalize(const FSSpec& inFileSpec, FSSpec* outSpec)
{
	ThrowIfOSErr_(::FSMakeFSSpec(inFileSpec.vRefNum, inFileSpec.parID, 
			inFileSpec.name, outSpec));
}

void WriteString(LStream& inStream, const string& theString)
{
	long stringLength = theString.length();
	inStream << stringLength;
	inStream.WriteBlock(theString.data(), stringLength);
	
}

void WriteStringAsText(LStream& inStream, const string& theString)
{
	long stringLength = theString.length();
	inStream.WriteBlock(theString.data(), stringLength);
}

string ReadString(LStream& inStream)
{
	long stringLength;
	inStream >> stringLength;
	StPointerBlock block(stringLength);
	inStream.ReadBlock(block, stringLength);
	return string(block, stringLength);
}

void
FillListBox(LListBox* theBox, const vector<string>& theStrings)
{
	ListHandle lh = theBox->GetMacListH();
	ThrowIfNil_(lh);
	int stringCount = theStrings.size();
	int colCount = ((**lh).dataBounds.right - (**lh).dataBounds.left);
	int rowCount = (stringCount + colCount - 1)/colCount;
	int delta = (**lh).dataBounds.bottom - rowCount;
	if (delta > 0) {
		::LDelRow(delta, 0, lh);
	} else if (delta < 0) {
		::LAddRow(-delta, 0, lh);
	}
	
	for (int i = 0; i < stringCount; i +=1 ) {
		Point theCell;
		theCell.h = i % colCount;
		theCell.v = i / colCount;
		::LSetCell(theStrings[i].data(), theStrings[i].length(), theCell, lh);
	}
}

string
ConvertErrorToString(ExceptionCode errorCode)
{
	switch(errorCode) {
	case kFieldNameTooLongError:
		return LoadString(kConverterErrorStrings, kFieldNameTooLongIndex);
		break;
	case kFieldDataTooLong:
		return LoadString(kConverterErrorStrings, kFieldDataTooLongIndex);
		break;
	case kFieldDataMissingQuote:
		return LoadString(kConverterErrorStrings, kFieldMissingQuoteIndex);
		break;
	case kTooManyFields:
		return LoadString(kConverterErrorStrings, kTooManyFieldsIndex);
		break;
	case kWrongVersion:
		return LoadString(kConverterErrorStrings, kVersionMismatchIndex);
		break;
	case kCantConvertFileError:
		return LoadString(kConverterErrorStrings, kFileCannotBeConvertedIndex);
		break;
	case kNotJTutorTextFileError:
		return LoadString(kConverterErrorStrings, kNotJTutorIndex);
		break;
	case kTooManyRecords:
		return LoadString(kConverterErrorStrings, kTooManyRecordsIndex);
		break;
	case kDatabaseTooLarge:
		return LoadString(kConverterErrorStrings, kDatabaseToLargeIndex);
		break;
	case kRecordDataTooLong:
		return LoadString(kConverterErrorStrings, kRecordTooLargeIndex);
		break;
	case kCantStartExisting:
		return LoadString(kConverterErrorStrings, kCantSyncExistingIndex);
		break;
	case kPopupChoicesTooLong:
		return LoadString(kConverterErrorStrings, kPopupsTooLargeIndex);
		break;
	case kNoFileMakerErr:
		return LoadString(kConverterErrorStrings, kNoFileMakerIndex);
		break;
	case insufficientStackErr:
		return LoadString(kConverterErrorStrings, kNoStackSpaceIndex);
		break;
	case fnfErr:
		return LoadString(kConverterErrorStrings, kFileCannotBeFoundIndex);
		break;
	case errAEInTransaction:
		return LoadString(kConverterErrorStrings, kInTransactionIndex);
		break;
	case memFullErr:
		return LoadString(kConverterErrorStrings, kOutOfMemoryIndex);
		break;
#ifdef CONDUIT
#ifdef DEMO
	case kDemoLimitExceeded:
		return LoadString(kFMJErrorStrings, kDemoLimitExceededIndex);
		break;
#endif
	case kLockFileInUse:
		return LoadString(kFMJErrorStrings, kLockFileInUseIndex);
		break;
	case SYNCERR_LOCAL_CANCEL_SYNC:
		return LoadString(kConverterErrorStrings, kLocalCancelIndex);
		break;
	case SYNCERR_LOST_CONNECTION:
		return LoadString(kConverterErrorStrings, kLostConnectionIndex);
		break;
	case SYNCERR_REMOTE_CANCEL_SYNC:
		return LoadString(kConverterErrorStrings, kRemoteCancelIndex);
		break;
	case SYNCERR_TOO_MANY_OPEN_FILES:
		return LoadString(kConverterErrorStrings, kRemoteTooManyFiles);
		break;
	case SYNCERR_REMOTE_MEM:
	case SYNCERR_REMOTE_NO_SPACE:
		return LoadString(kConverterErrorStrings, kRemoteOutOfMemory);
		break;
#endif
	case kUnknownError:
		return LoadString(kConverterErrorStrings, kUnexpectedErrorIndex);
		break;
	}
	string unknown(LoadString(kFMJErrorStrings, kErrorNumberIndex));
	unknown.append(AsString(errorCode));
	unknown.append(" [0x");
	unknown.append(HexAsString(errorCode));
	unknown.append("])");
	return unknown;
}

int cmp_nocase(const string &s, const string& s2)
{
	string::const_iterator p = s.begin();
	string::const_iterator p2 = s2.begin();
	
	while (p != s.end() && p2 != s2.end()) {
		if (toupper(*p) != toupper(*p2))
			return (toupper(*p) < toupper(*p2)) ? -1: 1;
		++p;
		++p2;
	}
	return s2.size() - s.size();
}
#ifdef _DEBUG
//#define LOG_STACK_SPACE 1
#endif

const int kMinFreeStackSpace = 4096;
void ConfirmFreeStackSpace()
{
#ifdef CONDUIT
	unsigned long stackFreeSpace;
	ThreadCurrentStackSpace(kCurrentThreadID, &stackFreeSpace);
	if (stackFreeSpace < kMinFreeStackSpace)
		Throw_(insufficientStackErr); 
#if LOG_STACK_SPACE
		DEBUG_OUT_NO_TS_DEF( "free stack space = " );
		DEBUG_OUT_NO_TS_DEF( AsString((int)stackFreeSpace).c_str());
		DEBUG_OUT_NO_TS_DEF( "\r" );
#endif
#endif
}

bool IsQuietError(ExceptionCode thisError)
{
	switch(thisError) {
	case kSyncUserCanceledErr:
	case kSyncQuietAbortErr:
		return true;
	}
	return false;
}

const ResIDT kFolderStrings = 8401;
const ResIDT kFileStrings = 8402;
enum { kFolderOnDiskIndex = 1, kDesktopFolderIndex, kRootFolderIndex};

string
DescribeFolder(const FSSpec& thisFolder)
{
	string folderIs;
	Str27 volName;
	short vRefNum;
	unsigned long freeBytes, totalBytes;
	ThrowIfOSErr_(HGetVInfo(thisFolder.vRefNum, volName, &vRefNum, &freeBytes, &totalBytes));
	long dirID;
	Boolean isDir;
	ThrowIfOSErr_(FSpGetDirectoryID(&thisFolder, &dirID, &isDir));
	ThrowIfNot_(isDir);
	if (thisFolder.parID == fsRtParID) {
		folderIs = LoadString(kFolderStrings, kRootFolderIndex);
		Substitute(folderIs, "%%2", AsString(volName));
	} else {
		long foundDirID;
		short foundVRefNum;
		ThrowIfOSErr_(FindFolder(vRefNum, kDesktopFolderType, false, &foundVRefNum, &foundDirID));
		if (foundDirID == dirID) {
			folderIs = LoadString(kFolderStrings, kDesktopFolderIndex);
			Substitute(folderIs, "%%2", AsString(volName));
		} else {
			folderIs = LoadString(kFolderStrings, kFolderOnDiskIndex);
			Substitute(folderIs, "%%1", AsString(thisFolder.name));
			Substitute(folderIs, "%%2", AsString(volName));
		}
	}
	return folderIs;
}

string
DescribeFile(const FSSpec& thisFile)
{
	FSSpec thisFolder;
	string folderIs;
	Str27 volName;
	short vRefNum;
	unsigned long freeBytes, totalBytes;
	ThrowIfOSErr_(HGetVInfo(thisFile.vRefNum, volName, &vRefNum, &freeBytes, &totalBytes));
	long dirID;
	Boolean isDir;
	ThrowIfOSErr_(FSMakeFSSpec(thisFile.vRefNum, thisFile.parID, 0, &thisFolder));
	ThrowIfOSErr_(FSpGetDirectoryID(&thisFolder, &dirID, &isDir));
	ThrowIfNot_(isDir);
	if (thisFolder.parID == fsRtParID) {
		folderIs = LoadString(kFileStrings, kRootFolderIndex);
		Substitute(folderIs, "%%3", AsString(volName));
	} else {
		long foundDirID;
		short foundVRefNum;
		ThrowIfOSErr_(FindFolder(vRefNum, kDesktopFolderType, false, &foundVRefNum, &foundDirID));
		if (foundDirID == dirID) {
			folderIs = LoadString(kFileStrings, kDesktopFolderIndex);
			Substitute(folderIs, "%%3", AsString(volName));
		} else {
			folderIs = LoadString(kFileStrings, kFolderOnDiskIndex);
			Substitute(folderIs, "%%2", AsString(thisFolder.name));
			Substitute(folderIs, "%%3", AsString(volName));
		}
	}
	Substitute(folderIs, "%%1", AsString(thisFile.name));
	return folderIs;
}

vector<string>
ExpandStringList(const vector<string>& labels, const vector<int>& repeats)
{
	vector<string> resultList;
	
	vector<string>::const_iterator i = labels.begin();
	vector<int>::const_iterator j = repeats.begin();
	while (i != labels.end()) {
		int repeatCount = *j;
		if (repeatCount > 1) {
			for (int index = 1; index <= repeatCount; index += 1)
				resultList.push_back(*i + " " + AsString(index));
		} else {
			resultList.push_back(*i);
		}
		++i;
		++j;
	}
	return resultList;
}

void WriteFileSpec(LStream& outStream, const FSSpec& aSpec)
{
	AliasHandle alias;
	ThrowIfOSErr_(::NewAlias(0, &aSpec, &alias));
	StHandleBlock hb((Handle)alias);
	outStream << (Handle)alias;
}

bool ReadFileSpec(LStream& inStream, FSSpec& aSpec)
{
	Handle h;
	inStream >> h;
	StHandleBlock hb(h);
	AliasHandle alias = (AliasHandle)h;
	Boolean wasChanged;
	int rulesMask = kARMNoUI | kARMSearch;
	short fileCount = 1;
	OSErr resolveErr = ::MatchAlias(0, rulesMask, alias, &fileCount, &aSpec, &wasChanged, 0, 0);
	return resolveErr == noErr;
}
