#include <Pilot.h>
#include <SysEvtMgr.h>
#include <SerialMgr.h>
#include "PalmJectorRsc.h"
#include "MouseData.h"
#include "PacketTypes.h"

/***********************************************************************
 *
 *   Declarations
 *
 ***********************************************************************/
 static char* ErrorMsg(const char *msg, int error);
 static void SendPacket(long packetType, const void *packetData, int length, Err* err);
 static Err RomVersionCompatible(DWord requiredVersion, Word launchFlags);
 static VoidPtr GetObjectPtr(Word objectID);
 static void MainFormInit(FormPtr frmP);
 static void AboutFormInit(FormPtr frmP);
 static Boolean MainFormDoCommand(Word command);
 static Boolean AboutFormHandleEvent(EventPtr eventP);
 static Boolean MainFormHandleEvent(EventPtr eventP);
 static Boolean AppHandleEvent( EventPtr eventP);
 static void AppEventLoop(void);
 static Err AppStart(void);
 static void AppStop(void);
 static DWord StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags);
 DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags);
 
/***********************************************************************
 *
 *	Macros
 *
 ***********************************************************************/
#define RandN(N) ((((long) SysRandom (0)) * (N)) / ((long) sysRandomMax + 1))

/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
struct StarterPreferenceType
{
	Byte replaceme;
} ;

struct StarterAppInfoType
{
	Byte replaceme;
} ;

/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
const unsigned long appFileCreator = 'STRT';
const unsigned short appVersionNum = 0x01;
const unsigned short appPrefID = 0x00;
const unsigned short appPrefVersionNum = 0x01;
const unsigned long version20 = 0x02000000;

#define screenWidth					160
#define screenHeight				145

/***********************************************************************
 *
 *   Internal Data Members
 *
 ***********************************************************************/
UInt		gSerRef = 0;

/***********************************************************************
 *
 * FUNCTION:    ErrorMsg
 *
 * DESCRIPTION: This routine creates a string error message from the error
 * number.
 *
 * PARAMETERS:  later
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 static char* ErrorMsg(const char *msg, int error)
 {
 	static char buffer[256];
 	static char number[256];
 	StrCopy(buffer, msg);
 	StrCat(buffer, " (");
 	SysErrString(error, number, 256);
 	StrCat(buffer, number);
 	StrCat(buffer, ")");
 	return buffer;
}

/***********************************************************************
 *
 * FUNCTION:    SendPacket
 *
 * DESCRIPTION: This routine sends a packet.
 *
 * PARAMETERS:  later
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void SendPacket(long packetType, const void *packetData, int length, Err* err)
{
	PacketHeader h;
	h.type = packetType;
	h.size = length;
	SerSend(gSerRef, &h, sizeof(h), err);
	//If there is a line error, clear it.
	if (*err == serErrLineErr)
		SerClearErr (gSerRef);
#if 0
	if (*err)
		return;
	SerSend(gSerRef, (void*)packetData, length, err);
	if (*err == serErrLineErr)
		SerClearErr (gSerRef);
#endif
}

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
static Err RomVersionCompatible(DWord requiredVersion, Word launchFlags)
{
	DWord romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < version20)
				{
//				Err err;
				
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return (sysErrRomIncompatible);
		}

	return (0);
}

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    VoidPtr
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static VoidPtr GetObjectPtr(Word objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();

	return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}

/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
}

/***********************************************************************
 *
 * FUNCTION:    AboutFormInit
 *
 * DESCRIPTION: This routine initializes the AboutForm form.
 *
 * PARAMETERS:  frm - pointer to the AboutForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AboutFormInit(FormPtr frmP)
{
}

/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(Word command)
{
	Boolean handled = false;

	switch (command)
	{
		case AboutAboutPalmJector:
			FrmGotoForm(AboutForm);
			handled = true;
			break;
	}
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    AboutFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "AboutForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AboutFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;
    
	switch (eventP->eType) 
	{
		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			AboutFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;

		case ctlSelectEvent:
			if (eventP->data.ctlSelect.controlID == AboutOKButtonButton)
			{
				FrmGotoForm(MainForm);
			}
		default:
			break;
		
		}
	
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;
    static gNextPacket = 0;
    
	switch (eventP->eType) 
	{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;

		case ctlSelectEvent:
			if (eventP->data.ctlSelect.controlID == MainEjectButton)
			{
				char c;
				Err err;
				SendPacket(1, &c, 0, &err);
				//ErrNonFatalDisplayIf(err, ErrorMsg("Error sending null packet", err));
				handled = true;
			}
			break;

		default:
			if (TimGetTicks() > gNextPacket) {
				char c;
				Err err;
				SendPacket(0, &c, 0, &err);
				gNextPacket = TimGetTicks() + 100;
			}
			break;
		
		}
	
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent( EventPtr eventP)
{
	Word formId;
	FormPtr frmP;


	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
		{
			case MainForm:
			{
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
			}
			
			case AboutForm:
			{
				FrmSetEventHandler(frmP, AboutFormHandleEvent);
				break;
			}

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
			break;

		}
		return true;
	}
	
	return false;
}

/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static void AppEventLoop(void)
{
	Word error;
	EventType event;
	
	do {
		EvtGetEvent(&event, 10);
		
		//Handle the mundainities
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}

/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
    StarterPreferenceType prefs;
    Word prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}

	//Initialize the Serial port
	SysLibFind("Serial Library", &gSerRef);
	Err err = SerOpen(gSerRef, 0, 57600);
	//ErrNonFatalDisplayIf(err, ErrorMsg("Error opening serial port", err));
	SerSettingsType serSet;
	serSet.baudRate = 57600;
	serSet.flags = serDefaultSettings | serSettingsFlagRTSAutoM | serSettingsFlagCTSAutoM;
	serSet.ctsTimeout = 900;
	err = SerSetSettings(gSerRef, &serSet);
	//ErrNonFatalDisplayIf(err, ErrorMsg("Setting up serial port", err));
	
   return 0;
}

/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	StarterPreferenceType prefs;  

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);

	SerClose(gSerRef);
}

/***********************************************************************
 *
 * FUNCTION:    StarterPilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static DWord StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Err error;

	error = RomVersionCompatible (version20, launchFlags);
	if (error) return (error);


	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return 0;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags)
{
    return StarterPilotMain(cmd, cmdPBP, launchFlags);
}
