// Unfinder.cp

#ifndef Unfinder_h
#include "Unfinder.h"
#endif
#ifndef RootFocus_h
#include "RootFocus.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif
#ifndef ProcessInfo_h
#include "ProcessInfo.h"
#endif
#ifndef __FINDERREGISTRY__
#include <FinderRegistry.h>
#endif
#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif
#ifndef SplitIntegers_h
#include "SplitIntegers.h"
#endif
#ifndef AEGetter_h
#include "AEGetter.h"
#endif
#ifndef FinderNamingLoop_h
#include "FinderNamingLoop.h"
#endif
#ifndef AEAddress_h
#include "AEAddress.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef NewFolderAction_h
#include "NewFolderAction.h"
#endif
#ifndef AppleEventRecorderSilencer_h
#include "AppleEventRecorderSilencer.h"
#endif
#ifndef AEDescriptor_h
#include "AEDescriptor.h"
#endif
#ifndef MakeAliasAction_h
#include "MakeAliasAction.h"
#endif
#ifndef DuplicateAction_h
#include "DuplicateAction.h"
#endif
#ifndef CopyAction_h
#include "CopyAction.h"
#endif
#ifndef RenameAction_h
#include "RenameAction.h"
#endif
#ifndef AEStruct_h
#include "AEStruct.h"
#endif
#ifndef MoveAction_h
#include "MoveAction.h"
#endif
#ifndef LockAction_h
#include "LockAction.h"
#endif
#ifndef DeleteAction_h
#include "DeleteAction.h"
#endif

Unfinder::Unfinder()
  : ActionLocus( RootFocus::The() )
  {
  }

Unfinder& Unfinder::The()
  {
	static Unfinder *initialized = 0;
	
	if ( initialized == 0 )
	  {
		AppleEventRecorderSilencer silencer;
		static Unfinder the;
		initialized = &the;
	  }

	return *initialized;
  }

FileSignature Unfinder::TargetSignature( const AEEvent& event )
  {
	ProcessSerialNumber processNumber =
		AEGetter<typeProcessSerialNumber>( event.Attribute( keyAddressAttr ) );

	return ProcessInfo( processNumber ).Signature();
  }

void Unfinder::Record( const AEEvent& event )
  {
	if ( TargetSignature( event ) != FileSignature::Finder() )
		return;
	
	AEVerb verb = event.Verb();
	
	switch ( verb.Suite().Suite() )
	  {
		case kCoreEventClass:		// The required suite
			break;

		case kAECoreSuite:
			RecordCoreEvent( verb, event );
			break;
		
		case kAEFinderSuite:
			RecordFinderEvent( verb, event );
			break;

		case kAEMiscStandards:
			RecordMiscellaneousEvent( verb, event );
			break;

		default:
			RecordUnknownEvent( verb, event );
			break;
	  }
  }

void Unfinder::RecordCoreEvent( AEVerb verb,
										  const AEEvent& event )
  {
	switch ( verb.Event() )
	  {
		case kAEClose:
			SetAction( true, "\pMove", Action::Irreversible() );
			break;

		case kAEDelete:
			RecordDelete( event );
			break;
		
		case kAEClone:
			RecordDuplicate( event );
			break;
		
		case kAECreateElement:
			RecordMake( event );
			break;
		
		case kAEMove:
			RecordMove( event );
			break;

		case kAESetData:
			RecordSetData( event );
			break;
		
		default:
			RecordUnknownEvent( verb, event );
			break;
	  }
  }

void Unfinder::RecordFinderEvent( AEVerb verb, 
											 const AEEvent& event )
  {
	switch ( verb.Event() )
	  {
		case kAERestart:
		case kAEShutDown:
		case kAESleep:
			break;
		
		case 'ffav':
			SetAction( true, "\pAdd To Favorites", Action::Irreversible() );
			break;

		case kAECleanUp:
			break;

		case kAEEject:
			SetAction( true, "\pEject Disk", Action::Irreversible() );
			break;

		case kAEEmpty:
			SetAction( true, "\pEmpty Trash", Action::Irreversible() );
			break;

		case kAEErase:
			SetAction( true, "\pErase Disk", Action::Irreversible() );
			break;

		case kAEPutAway:
			SetAction( true, "\pPut Away", Action::Irreversible() );
			break;
		
		default:
			RecordUnknownEvent( verb, event );
			break;
	  }
  }

void Unfinder::RecordMiscellaneousEvent( AEVerb verb,
													  const AEEvent& event )
  {
	switch ( verb.Event() )
	  {
		case kAESelect:
			RecordSelect( event );
			break;

		case kAECopy:
			break;
		
		default:
			RecordUnknownEvent( verb, event );
			break;
	  }
  }

void Unfinder::RecordUnknownEvent( AEVerb verb, const AEEvent& )
  {
	uint8 name[10];
	name[0] = 9;
	name[1] = Byte3( verb.Suite().Suite() );
	name[2] = Byte2( verb.Suite().Suite() );
	name[3] = Byte1( verb.Suite().Suite() );
	name[4] = Byte0( verb.Suite().Suite() );
	name[5] = '/';
	name[6] = Byte3( verb.Event() );
	name[7] = Byte2( verb.Event() );
	name[8] = Byte1( verb.Event() );
	name[9] = Byte0( verb.Event() );

	SetAction( true, name, Action::Irreversible() );
	Assert( 0 );
  }

void Unfinder::RecordMake( const AEEvent& event )
  {
	DescType typeCreated = AEGetter<typeType>( event.Parameter( keyAEObjectClass ) );
	
	switch ( typeCreated )
	  {
		case cFolder:
		  {
			CommitAction();
			static NewFolderAction action;
			action.SetEvent( event );
			SetAction( true, "\pMake Folder", action );
			break;
		  }
		
		case cAliasFile:
		  {
			CommitAction();
			static MakeAliasAction action;
			action.SetEvent( selection, event );
			SetAction( true, "\pMake Alias", action );
			break;
		  }
	  }
  }

void Unfinder::RecordMove( const AEEvent& event )
  {
	static MoveAction action;
	action.SetEvent( selection, event );
	SetAction( true, "\pMove", action );
  }

void Unfinder::RecordDelete( const AEEvent& event )
  {
	static DeleteAction action;
	action.SetEvent( selection, event );
	SetAction( true, "\pDelete", action );
  }

void Unfinder::RecordDuplicate( const AEEvent& event )
  {
	CommitAction();
	
	if ( event.Parameter( keyAEInsertHere ).Exists() )
	  {
		static CopyAction action;
		action.SetEvent( selection, event );
		SetAction( true, "\pCopy", action );
	  }
	 else
	  {
		static DuplicateAction action;
		action.SetEvent( selection, event );
		SetAction( true, "\pDuplicate", action );
	  }
  }

void Unfinder::RecordSetData( const AEEvent& event )
  {
	AEStruct directObject( event.Parameter( keyDirectObject ) );
	
	if ( AEGetter<typeEnumerated>( directObject[ 'form' ] ) != keyAEPropID )
		return;
	
	DescType property = AEGetter<typeType>( directObject[ 'seld' ] );
	
	switch ( property )
	  {
		case pName:
		  {
			CommitAction();
			static RenameAction action;
			action.SetEvent( selection, event );
			SetAction( true, "\pName Change", action );
			break;
		  }

		case pIsLocked:
		  {
			CommitAction();
			static LockAction action;
			action.SetEvent( event );
			SetAction( true,
						  action.Locking() ? "\pLock" : "\pUnlock",
						  action );
			break;
		  }
	  }
  }

void Unfinder::RecordSelect( const AEEvent& event )
  {
	AEDescriptor newSelection( event.Parameter( keyDirectObject ), typeAEList );
	
	selection.Swap( newSelection );
  }

